#include "config.h"

#include <gtk/gtk.h>

#include "boilerplate-application.h"
#include "boilerplate-window.h"

typedef struct
{
    GSettings *settings;

    GtkWidget *headerbar;
    GtkWidget *toolbar;
    GtkWidget *content;
} BoilerplateWindowPrivate;

G_DEFINE_TYPE_WITH_PRIVATE(BoilerplateWindow, boilerplate_window, GTK_TYPE_APPLICATION_WINDOW)

/**
 * boilerplate_window_init:
 * @application: The application
 *
 * Initializes the application window.
 *
 * Returns: BoilerplateWindow -> GtkApplicationWindow
 *
 * Since: 0.1.0
 */
static void
boilerplate_window_init (BoilerplateWindow *self)
{
    BoilerplateApplication *application = BOILERPLATE_APPLICATION (g_application_get_default ());
    BoilerplateWindowPrivate* priv = boilerplate_window_get_instance_private (self);
    GtkBuilder *builder;

    /* Center window on screen */
    //TODO: Fix & Load last location from gsettings
    gtk_window_set_position (GTK_WINDOW (self), GTK_WIN_POS_CENTER);

    /* Init window.ui template */
    gtk_widget_init_template (GTK_WIDGET (self));

    /* Adding headerbar/titlebar/toolbar */
    builder = gtk_builder_new_from_resource (APPLICATION_RESOURCES "/ui/toolbar.ui");
    priv->toolbar = GTK_WIDGET (gtk_builder_get_object (builder, "toolbar"));

    /*
     * If CSD is disabled we load the toolbar/headerbar into the window's main
     * area, we also apply a small amoutn of padding to make it look like a
     * toolbar. Otherwise we load the GtkHeaderbar/titlebar.
     */
    //TODO: Should be able to gtk_application_window_get_show_menubar (GtkApplicationWindow)
    if (application->disable_csd) {
        gtk_container_add (GTK_CONTAINER (priv->content), priv->toolbar);

        gtk_widget_set_margin_start (GTK_WIDGET (priv->toolbar), 7);
        gtk_widget_set_margin_end (GTK_WIDGET (priv->toolbar), 7);
        gtk_widget_set_margin_top (GTK_WIDGET (priv->toolbar), 7);
        gtk_widget_set_margin_bottom (GTK_WIDGET (priv->toolbar), 7);
    } else {
        gtk_window_set_titlebar (GTK_WINDOW (self), priv->headerbar);
        gtk_container_add (GTK_CONTAINER (priv->headerbar), priv->toolbar);
    }

    g_object_unref (builder);
}

/**
 * boilerplate_window_class_init:
 * @class: The window class
 *
 * Initializes the various functions in the window and links the gtk_widget
 * template loaded from gresource.
 *
 * Returns: void
 *
 * Since: 0.1.0
 */
static void
boilerplate_window_class_init (BoilerplateWindowClass *klass)
{
    gtk_widget_class_set_template_from_resource (GTK_WIDGET_CLASS (klass),
                                                 APPLICATION_RESOURCES "/ui/window.ui");

    gtk_widget_class_bind_template_child_private (GTK_WIDGET_CLASS (klass), BoilerplateWindow, headerbar);
    gtk_widget_class_bind_template_child_private (GTK_WIDGET_CLASS (klass), BoilerplateWindow, content);
}

/**
 * boilerplate_window_new:
 * @application: The application
 *
 * Creates a new GtkApplicationWindow.
 *
 * Returns: BoilerplateWindow -> GtkApplicationWindow
 *
 * Since: 0.1.0
 */
BoilerplateWindow *
boilerplate_window_new (BoilerplateApplication *application)
{
    return g_object_new (BOILERPLATE_TYPE_WINDOW,
                         "application", application,
                         "title", APPLICATION_NAME,
                         "show-menubar", application->disable_csd ? TRUE : FALSE,
                         NULL);
}
